// MkIII music box version 8 code
// Credit to https://www.hackster.io/circuiTician/i-made-an-arduino-based-music-player-using-dfplayer-mini-c3c7c9
// For the code tips and the concepts. Transmission through Rx and Tx modified accordingly
// Additional code by Neil Williamson from discussions with Tim King for ideas and concepts and generally 
// putting up with me during the idea generation process. To my daughter Eva - in all likelihood you'll
// never see this. Despite all I have achieved in this world, you are my greatest gift. My only hope is that you
// grow into the person you deserve to be. Love dad.

#include "Arduino.h"
#include "SoftwareSerial.h"
#include "DFRobotDFPlayerMini.h"
 
SoftwareSerial mySoftwareSerial(4, 3); // RX, TX
DFRobotDFPlayerMini myDFPlayer;
void printDetail(uint8_t type, int value);
 
#define pp 8
#define vu 9
#define vd 5
#define nxt 6
#define prv 7


//Set start volume for speaker from 10 to 30 - 10 is OK for room use, 20 is loud!
int v= 20, pv=0;

// Setting up a pulse to keep the battery bank powered up.

// this constant won't change. It's the pin number of the 47 Ohm resistor:
const int PulsePin =  10; // the number of the Pulse output
// PulseState used to set the pulse
int PulseState = LOW;
// Generally, you should use "unsigned long" for variables that hold time
// The value will quickly become too large for an int to store
unsigned long previousMillis = 0;        // will store last time Pulse was updated
// constants won't change:
const long interval = 1000;           // interval at which to send pulse (milliseconds)

void setup()
{
  mySoftwareSerial.begin(9600);
  Serial.begin(115200);
 
  pinMode(pp,INPUT_PULLUP);
  pinMode(nxt,INPUT_PULLUP);
  pinMode(prv,INPUT_PULLUP);
  pinMode(vu,INPUT_PULLUP);
  pinMode(vd,INPUT_PULLUP);

  // set the digital pin as output:
  pinMode(PulsePin, OUTPUT);
  
  Serial.println();
  Serial.println(F("DFRobot DFPlayer Mini Demo"));
  Serial.println(F("Initializing DFPlayer ... (May take 3~5 seconds)"));
  
  if (!myDFPlayer.begin(mySoftwareSerial)) {  //Use softwareSerial to communicate with mp3.
    Serial.println(F("Unable to begin:"));
    Serial.println(F("1.Please recheck the connection!"));
    Serial.println(F("2.Please insert the SD card!"));
// Note: For some reason the program hangs when installed on generic Nano's so trying commenting out this code
//    while(true){
//      delay(0); // Code to compatible with ESP8266 watch dog.
//    }
  }
  Serial.println(F("DFPlayer Mini online."));
  
  myDFPlayer.volume(v);  //Sets volume value. From 0 to 30
  //myDFPlayer.play(1);  //Play the first mp3
}
 
void loop()
{
  Serial.println(pv);

  // Pulse to sustain power bank output through PulsePin
  // check to see if it's time to pulse; that is, if the difference
  // between the current time and last time the pulse was made is bigger than
  // the interval at which you want to send the pulse.
  unsigned long currentMillis = millis();

  if (currentMillis - previousMillis >= interval) {
    // save the last time the pulse was sent
    previousMillis = currentMillis;

    // if the pulse is off turn it on and vice-versa:
    if (PulseState == LOW) {
      PulseState = HIGH;
    } else {
      PulseState = LOW;
    }
    // set the pulse with the PulseState of the variable:
    digitalWrite(PulsePin, PulseState);
  }
  //This is the element that can be edited to add play for x time then pause
  if(digitalRead(pp)==0){
    if(pv == 0){
    myDFPlayer.start();
    delay(10000);
    myDFPlayer.pause();
    pv=1;
    }
    else if(pv == 1){
    myDFPlayer.pause();
    pv=0;
    }
  }
  //If pause code not added, next track will play but not pause.
  if(digitalRead(nxt)==0){
    myDFPlayer.next();
    delay(2000);
    myDFPlayer.pause();
  }
  if(digitalRead(prv)==0){
    myDFPlayer.previous();
    
  }
  if(digitalRead(vu)==0 && v<30){
    v=v+1;
    myDFPlayer.volume(v);
  }
  if(digitalRead(vd)==0 && v>0){
    v=v-1;
    myDFPlayer.volume(v);
  }
  delay(100);
  
  if (myDFPlayer.available()) {
    printDetail(myDFPlayer.readType(), myDFPlayer.read()); //Print the detail message from DFPlayer to handle different errors and states.
  }
}
 
void printDetail(uint8_t type, int value){
  switch (type) {
    case TimeOut:
      Serial.println(F("Time Out!"));
      break;
    case WrongStack:
      Serial.println(F("Stack Wrong!"));
      break;
    case DFPlayerCardInserted:
      Serial.println(F("Card Inserted!"));
      break;
    case DFPlayerCardRemoved:
      Serial.println(F("Card Removed!"));
      break;
    case DFPlayerCardOnline:
      Serial.println(F("Card Online!"));
      break;
    case DFPlayerUSBInserted:
      Serial.println("USB Inserted!");
      break;
    case DFPlayerUSBRemoved:
      Serial.println("USB Removed!");
      break;
    case DFPlayerPlayFinished:
      Serial.print(F("Number:"));
      Serial.print(value);
      Serial.println(F(" Play Finished!"));
      break;
    case DFPlayerError:
      Serial.print(F("DFPlayerError:"));
      switch (value) {
        case Busy:
          Serial.println(F("Card not found"));
          break;
        case Sleeping:
          Serial.println(F("Bored now - sleeping"));
          break;
        case SerialWrongStack:
          Serial.println(F("Get Wrong Stack"));
          break;
        case CheckSumNotMatch:
          Serial.println(F("Check Sum Not Match"));
          break;
        case FileIndexOut:
          Serial.println(F("File Index Out of Bound"));
          break;
        case FileMismatch:
          Serial.println(F("Cannot Find File"));
          break;
        case Advertise:
          Serial.println(F("In Advertise"));
          break;
        default:
          break;
      }
      break;
    default:
      break;
  }
  
}
